<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2017, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Backend;

/**
 * Class file
 * TableContent
 */
class TableContent extends \Contao\Backend
{	
	/**
	 * Import the back end user object
	 */
	public function __construct()
	{
		parent::__construct();
		$this->import('BackendUser', 'User');
	}


	/**
	 * Modify the DCA
	 * @param object
	 * 
	 * called from onload_callback
	 */
	public function modifyDCA($objDC)
	{
		if( $objDC->activeRecord === null )
		{
			$objDC->activeRecord = \Contao\ContentModel::findByPk($objDC->id);
		}
		
		$objCustomElemnts = new \PCT\CustomElements\Core\CustomElements;
		
		$strSelectionField = $objCustomElemnts->getSelectionField($objDC->table);
		$arrCustomElements = $objCustomElemnts->getCustomElements($objDC->table);
		if( \in_array($objDC->activeRecord->{$strSelectionField}, $arrCustomElements) )
		{
			$GLOBALS['TL_DCA'][$objDC->table]['fields']['customTpl']['options_callback'] = array('PCT\CustomElements\Backend\BackendIntegration','getTemplates');
		}
	}
	
	
	/**
	 * Add the set export button to the multi select formular buttons
	 * @param array
	 * @return array
	 * Called from buttons_callback Hook
	 */
	public function buttonsCallback($arrButtons)
	{
		if(!$this->User->isAdmin)
		{
			return $arrButtons;
		}
		
		$arrButtons['contentelementset_export'] = '<input type="submit" name="contentelementset_export" id="contentelementset_export" class="tl_submit contentelementset_export" accesskey="e" value="'.$GLOBALS['TL_LANG']['MSC']['contentelementsets_export'].'">';
		
		return $arrButtons;
	}
	
	
	/**
	 * Export the selected content elements
	 * @param object
	 * Called from loadDataContainer Hook
	 */
	public function exportElements($strTable)
	{
		if($strTable != 'tl_content' || empty($_POST['IDS']))
		{
			return;
		}
		
		$objContentModels = \ContentModel::findMultipleByIds( \Contao\Input::post('IDS') );
		if($objContentModels === null)
		{
			return;
		}
		
		// find all installed content element CustomElements
		$arrCustomElements = \PCT\CustomElements\Core\CustomElements::getInstance()->getCustomElements($strTable);
		
		$arrExport = array();
		foreach($objContentModels as $objModel)
		{
			$arrRow = $objModel->row();
			
			$arrExport['tl_content'][] = $arrRow;
			
			// tl_pct_customelement_vault
			if(in_array($objModel->type,$arrCustomElements))
			{
				$arrWizardData = \PCT\CustomElements\Core\Vault::getWizardData($objModel->id,$strTable);
				if(!empty($arrWizardData))
				{
					$arrExport['tl_pct_customelement_vault'][] = $arrWizardData;
				}
			}
		}
	}
}
