<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2014, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Core\Maintenance;

/**
 * Imports
 */
use PCT\CustomElements\Core\GroupFactory as GroupFactory;
use PCT\CustomElements\Core\AttributeFactory as AttributeFactory;
use PCT\CustomElements\Core\CustomElements as CustomElements;
use PCT\CustomElements\Core\Vault;

/**
 * Class file
 * AttributeUpdater
 */
class AttributeUpdater extends \Contao\Backend
{
	/**
	 * Job name
	 */
	protected $strJob = 'attributeupdater';
	
	
	/**
	 * Return true if the module is active
	 * @return boolean
	 */
	public function isActive()
	{
		if(\Contao\Input::get('key') == $this->strJob)
		{
			return true;
		}
		
		return false;
	}
	
	
	/**
	 * Run the update routines
	 * @return boolean
	 * Idea taken from \RebuildIndex
	 */
	public function run()
	{
		if(!\RequestToken::validate(\Contao\Input::get('rt')) && !\RequestToken::validate(\Contao\Input::post('REQUEST_TOKEN')))
		{
		  $this->Session->set('INVALID_TOKEN_URL', \Contao\Environment::get('request'));
		  $this->redirect('contao/confirm.php');
		}
					
		// update a certain wizard
		$vault_id = \Contao\Input::get('vault_id');
		if($vault_id  > 0)
		{
			$objRow = \Contao\Database::getInstance()->prepare("SELECT * FROM tl_pct_customelement_vault WHERE id=?")
							->limit(1)
							->execute($vault_id);
			if($objRow->numRows > 0)
			{
				$objAttribute = AttributeFactory::findById($objRow->attr_id);
				if(!$objAttribute) {return;}
				
				$from = $GLOBALS['PCT_CUSTOMELEMENTS']['MAINTENANCE'][$this->strJob]['types'][$objRow->type]['from'] ?: 'data';
 				$fromData = $objRow->{$from};
 				
 				if(strlen($GLOBALS['PCT_CUSTOMELEMENTS']['MAINTENANCE'][$this->strJob]['types'][$objRow->type]['to']) > 0)
 				{
	 				$objAttribute->set('saveDataAs',$to);
	 			}
 				$arrOptions = array('saveDataAs'=>$objAttribute->get('saveDataAs'));
 				$arrOptions['bypassCallback'] = true; 
 				
 				// store the new value
 				Vault::setAttributeValue($fromData,$objRow->pid,$objRow->attr_uuid,$objRow->source,$arrOptions);
 				return true;
			}
			else
			{
				$this->Session->set('REBUILD_INDEX_ERROR','Wrong or obsolete Vault data');
				return false;
			}
		}
		
		// render the page
		$objTemplate = new \Contao\BackendTemplate('be_maintenance_attributeupdater');
		$objTemplate->action = \ampersand(\Contao\Environment::get('request'));
		$objTemplate->isActive = $this->isActive();
		$objTemplate->indexContinue = $GLOBALS['TL_LANG']['MSC']['continue'];
		$objTemplate->theme = \Contao\Backend::getTheme();
		$objTemplate->note = $GLOBALS['TL_LANG']['tl_maintenance']['indexNote'];
		
		// Add the error message
		if ($this->Session->get('REBUILD_INDEX_ERROR') != '')
		{
			$objTemplate->indexMessage = $this->Session->get('REBUILD_INDEX_ERROR');
			$this->Session->set('REBUILD_INDEX_ERROR','');
		}
	
		$arrTypes = array_keys($GLOBALS['PCT_CUSTOMELEMENTS']['MAINTENANCE'][$this->strJob]['types']) ?: array();
	
		$arrAttributeData = $this->findAttributeData($arrTypes);
		if(empty($arrAttributeData))
		{
			$objTemplate->indexMessage = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['attributeupdater']['empty'];
			$objTemplate->isRunning = false;	
			return $objTemplate->parse();
		}
		
		$objDatabase = \Contao\Database::getInstance();
		// display the wizard information
		for ($i=0, $c=count($arrAttributeData); $i<$c; $i++)
		{
			$objAttribute = $objDatabase->prepare("SELECT * FROM tl_pct_customelement_attribute WHERE id=?")->limit(1)->execute($arrAttributeData[$i]['attr_id']);
			
			// fetch the source entry
			$title = $arrAttributeData[$i]['source'] .' ['.$arrAttributeData[$i]['pid'].'] => '.$objAttribute->title . ' ['.$objAttribute->type.']';
			$strBuffer .= '<span class="page_url vault_data" data-vault_id="'.$arrAttributeData[$i]['id'].'">' . $title . '</span><br>';
			unset($arrAttributeData[$i]);
		}
		
		$objTemplate->content = $strBuffer;
		$objTemplate->loading = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['attributeUpdateLoading'];
		$objTemplate->complete = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['attributeUpdateComplete'];
		$objTemplate->isRunning = true;
		
		return $objTemplate->parse();
	}
	
	
	/**
	 * Find attributes in vault and return data as array
	 * @return array
	 */
	public function findAttributeData($arrTypes)
	{
		$objDatabase = \Contao\Database::getInstance();
		$objVault = $objDatabase->prepare("SELECT * FROM tl_pct_customelement_vault WHERE type!='wizard' ".(count($arrTypes) > 0 ? ' AND '.$objDatabase->findInSet('type',$arrTypes) : '')." ORDER BY type,source,pid")->execute();
		if($objVault->numRows < 1)
		{
			return array();
		}
		return $objVault->fetchAllAssoc();
	}

	
	
	/**
	 * Update an existing wizard data structure to the latest one
	 * @param array
	 * @param integer
	 * @param string
	 * @return array
	 */
	public function updateWizard($arrData,$intPid,$strSource)
	{
		if(empty($arrData))
		{
			return array();
		}
		
		$this->pid = $intPid;
		$this->source = $strSource;
		
		$objVault = Vault::getInstance();
		
		$values = array();
		// add only uuids to the data
		if(is_array($arrData['groups']) && $arrData['alias'])
		{
			$uuids = array();
			foreach($arrData['groups'] as $group)
			{
				foreach($group['attributes'] as $attr)
				{
					$uuids[] = $attr['uuid'];
					
					// save value in wizard
					if($GLOBALS['PCT_CUSTOMELEMENTS']['saveValuesInWizard'])
					{
						$objAttribute = AttributeFactory::findById($attr['id']);
						if(!$objAttribute) {continue;}
						// fetch the value and put it in the values key
						$value = $objVault::getValue($attr['uuid'],$intPid,$strSource,array('saveDataAs'=>$objAttribute->get('saveDataAs')) );
						$values[$attr['uuid']] = $value;
					}
				}
			}
			$arrData['uuids'] = $uuids;
			$arrData['version'] = PCT_CUSTOMELEMENTS_VERSION;
			$arrData['values'] = $values;
			
			return $arrData;
		}
		
		// update the old structure from 1.1.6 and earlier
		else if(!$arrData['groups'] || !is_array($arrData['groups']))
		{
			$tstamp = $arrData['tstamp'];
			if(array_key_exists('tstamp', $arrData))
			{
				unset($arrData['tstamp']);
			}
			$new = array('groups' => $arrData, 'tstamp' => $tstamp);
			
			// find the corresponding custom element
			$selectionField = CustomElements::getSelectionField($this->source);
			$objElement = null;
			if(strlen($selectionField) > 0)
			{
				$objElement = \Contao\Database::getInstance()->prepare("SELECT ".$selectionField." FROM ".$this->source." WHERE id=?")->limit(1)->execute($this->pid);
				$new['alias'] = $objElement->$selectionField;
			}
			return $this->updateWizard($new,$intPid,$strSource);
		}
		
		return $arrData;
	}	
	
}