<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2014, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_customelements
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace PCT\CustomElements\Core\Maintenance;

/**
 * Imports
 */
use PCT\CustomElements\Core\GroupFactory as GroupFactory;
use PCT\CustomElements\Core\AttributeFactory as AttributeFactory;
use PCT\CustomElements\Core\CustomElements as CustomElements;
use PCT\CustomElements\Core\Vault;

/**
 * Class file
 * WizardUpdater
 */
class WizardUpdater extends \Contao\Backend
{
	/**
	 * Job name
	 */
	protected $strJob = 'wizardupdater';
	
	
	/**
	 * Return true if the module is active
	 * @return boolean
	 */
	public function isActive()
	{
		if(\Contao\Input::get('key') == $this->strJob)
		{
			return true;
		}
		
		return false;
	}
	
	
	/**
	 * Run the update routines
	 * @return boolean
	 * Idea taken from \RebuildIndex
	 */
	public function run()
	{
		if(!\RequestToken::validate(\Contao\Input::get('rt')) && !\RequestToken::validate(\Contao\Input::post('REQUEST_TOKEN')))
		{
		  $this->Session->set('INVALID_TOKEN_URL', \Contao\Environment::get('request'));
		  $this->redirect('contao/confirm.php');
		}
		
		// update a certain wizard
		$vault_id = \Contao\Input::get('vault_id');
		if($vault_id  > 0)
		{
			$objWizard = \Contao\Database::getInstance()->prepare("SELECT * FROM tl_pct_customelement_vault WHERE id=?")
							->limit(1)
							->execute($vault_id);
			
			if($objWizard->numRows > 0)
			{
				$arrData = \deserialize($objWizard->data_blob);
				if(empty($arrData))
				{
					return;
				}
			
				$pid = $objWizard->pid;
				$source = $objWizard->source;
				$data = $this->updateWizard($arrData,$pid,$source);
				
				Vault::saveWizardData($data,$pid,$source);
				return true;
			}
			else
			{
				$this->Session->set('REBUILD_INDEX_ERROR','Wrong or obsolete Vault data');
				return false;
			}
		}
		
		// render the page
		$objTemplate = new \Contao\BackendTemplate('be_maintenance_wizardupdater');
		$objTemplate->action = \ampersand(\Contao\Environment::get('request'));
		$objTemplate->isActive = $this->isActive();
		$objTemplate->indexContinue = $GLOBALS['TL_LANG']['MSC']['continue'];
		$objTemplate->theme = \Contao\Backend::getTheme();
		$objTemplate->note = $GLOBALS['TL_LANG']['tl_maintenance']['indexNote'];
		
		// Add the error message
		if ($this->Session->get('REBUILD_INDEX_ERROR') != '')
		{
			$objTemplate->indexMessage = $this->Session->get('REBUILD_INDEX_ERROR');
			$this->Session->set('REBUILD_INDEX_ERROR','');
		}
	
		$arrWizards = $this->findWizards();
		if(empty($arrWizards))
		{
			$objTemplate->indexMessage = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['wizardupdater']['empty'];
			$objTemplate->isRunning = false;	
			return $objTemplate->parse();
		}
		
		$objDatabase = \Contao\Database::getInstance();
		// display the wizard information
		for ($i=0, $c=count($arrWizards); $i<$c; $i++)
		{
			// find the corresponding custom element
			$selectionField = CustomElements::getSelectionField($arrWizards[$i]['source']);
			$objCE = null;
			if(strlen($selectionField) > 0)
			{
				$objCE = $objDatabase->prepare("SELECT * FROM tl_pct_customelement WHERE alias=(SELECT ".$selectionField." FROM ".$arrWizards[$i]['source']." WHERE id=?)")->limit(1)->execute($arrWizards[$i]['pid']);
			}
			// fetch the source entry
			$title = $arrWizards[$i]['source'] .' ['.$arrWizards[$i]['pid'].'] => '.$objCE->title . ' ['.$objCE->alias.']';
			$strBuffer .= '<span class="page_url wizard_data" data-vault_id="'.$arrWizards[$i]['id'].'">' . $title . '</span><br>';
			unset($arrWizards[$i]);
		}
		
		$objTemplate->content = $strBuffer;
		$objTemplate->loading = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['wizardUpdateLoading'];
		$objTemplate->complete = $GLOBALS['TL_LANG']['tl_maintenance']['pct_customelements']['wizardUpdateComplete'];
		$objTemplate->isRunning = true;
		
		return $objTemplate->parse();
	}
	
	
	/**
	 * Find wizards and return data as array
	 * @return array
	 */
	public function findWizards()
	{
		$objWizards = \Contao\Database::getInstance()->prepare("SELECT * FROM tl_pct_customelement_vault WHERE type=? ORDER BY source,pid")->execute('wizard');
		if($objWizards->numRows < 1)
		{
			return array();
		}
		
		return $objWizards->fetchAllAssoc();
	}

	
	
	/**
	 * Update an existing wizard data structure to the latest one
	 * @param array
	 * @param integer
	 * @param string
	 * @return array
	 */
	public function updateWizard($arrData,$intPid,$strSource)
	{
		if(empty($arrData))
		{
			return array();
		}
		
		$this->pid = $intPid;
		$this->source = $strSource;
		
		$objVault = Vault::getInstance();
		
		$values = array();
		// add only uuids to the data
		if(is_array($arrData['groups']) && $arrData['alias'])
		{
			$uuids = array();
			foreach($arrData['groups'] as $i => $group)
			{
				// include the group alias if it doesnt exists yet
				if(strlen($group['group_alias']) < 1 || !isset($group['group_alias']))
				{
					$objGroup = \Contao\Database::getInstance()->prepare("SELECT alias FROM tl_pct_customelement_group WHERE id=?")->limit(1)->execute($group['group_id']);
					$arrData['groups'][$i]['group_alias'] = $objGroup->alias;
				}
				
				$groupId = $group['group_id'];
				$groupAlias = $group['group_alias'] ?: $GLOBALS['PCT_CUSTOMELEMENTS']['cache']['group'][$groupId]['alias'];
			
				$isCopy = false;
				if($group['isCopy'] || $group['is_copy'])
				{
					$isCopy = true;
				}
			
				$objGroup = null;
				
				if(strlen($groupAlias) > 0)
				{
					$objGroup = GroupFactory::findByAliasAndCustomElement($groupAlias,$arrData['alias']);
				}
				// hardcore fallback to first CE version
				else if($groupId > 0 && strlen($groupAlias) < 1)
				{
					$objGroup = GroupFactory::findById($groupId);
				}
			
				if($objGroup !== null)
				{
					if($group['group_id'] != $objGroup->get('id'))
					{
						$arrData['groups'][$i]['group_id'] = $objGroup->get('id');
						$arrData['groups'][$i]['ident'] = $objGroup->get('id').'_0';
					}
					
					if($group['numCopy'] > 0)
					{
						$arrData['groups'][$i]['ident'] = $objGroup->get('id').'_'.$group['numCopy'];
						$group['pgroup'] = $objGroup->get('id');
					}
				}
				
				foreach($group['attributes'] as $attr)
				{
					$uuids[] = $attr['uuid'];
					
					// save value in wizard
					if($GLOBALS['PCT_CUSTOMELEMENTS']['saveValuesInWizard'])
					{
						$objAttribute = AttributeFactory::findByUuidAndCustomElement($attr['uuid'],$arrData['alias']);
						
						if($objAttribute === null && $attr['alias'])
						{
							$tmp = explode('#',  $attr['alias']);
							$alias = $tmp[0];
							
							$objAttribute = AttributeFactory::findByAlias($alias);
						}
						
						if($objAttribute === null && $attr['id'])
						{
							$objAttribute = AttributeFactory::findById($attr['id']);
						}
						
						
						if($group['is_copy'])
						{
							$objAttribute->uuid = $attr['uuuid'];
						}
						
						if(!$objAttribute) {continue;}
						// fetch the value and put it in the values key
						$value = $objVault::getValue($attr['uuid'],$intPid,$strSource,array('saveDataAs'=>$objAttribute->get('saveDataAs')) );
						
						if($objAttribute->get('saveDataAs') == 'binary')
						{
							if(class_exists('\Contao\StringUtil'))
							{
								$value = \Contao\StringUtil::binToUuid($value);
							}
							else
							{
								$value = \Contao\StringUtil::binToUuid($value);
							}
						}
						
						$values[$attr['uuid']] = $value;
						
						// generic options
						$arrOptions = \deserialize($objAttribute->get('options'));
						if(!empty($arrOptions) && is_array($arrOptions) && !in_array($objAttribute->type, $GLOBALS['PCT_CUSTOMELEMENTS']['ignoreOptionFields']))
						{
							foreach($arrOptions as $option)
							{
								$field = $attr['uuid'].'_'.$option;
								$values[$field] = \deserialize($objVault::getValue($field,$intPid,$strSource,array('saveDataAs'=>$objAttribute->get('saveDataAs')) ));
							}
						}
						
					}
				}
			}
			
			$arrData['uuids'] = $uuids;
			$arrData['version'] = PCT_CUSTOMELEMENTS_VERSION;
			$arrData['values'] = $values;
			
			return $arrData;
		}
		
		// update the old structure from 1.1.6 and earlier
		else if(!$arrData['groups'] || !is_array($arrData['groups']))
		{
			$tstamp = $arrData['tstamp'];
			if(array_key_exists('tstamp', $arrData))
			{
				unset($arrData['tstamp']);
			}
			$new = array('groups' => $arrData, 'tstamp' => $tstamp);
			
			// find the corresponding custom element
			$selectionField = CustomElements::getSelectionField($this->source);
			$objElement = null;
			if(strlen($selectionField) > 0)
			{
				$objElement = \Contao\Database::getInstance()->prepare("SELECT ".$selectionField." FROM ".$this->source." WHERE id=?")->limit(1)->execute($this->pid);
				$new['alias'] = $objElement->$selectionField;
			}
			return $this->updateWizard($new,$intPid,$strSource);
		}
		
		return $arrData;
	}	
	
}