<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2015
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_iconpicker
 * @link		http://contao.org
 */
 
namespace Contao;

/**
 * Class file
 * BackendIconPicker
 */
class BackendIconPicker extends \Contao\Backend
{
	/**
	 * Current Ajax object
	 * @var object
	 */
	protected $objAjax;

	/**
	 * Initialize the controller
	 *
	 * 1. Import the user
	 * 2. Call the parent constructor
	 * 3. Authenticate the user
	 * 4. Load the language files
	 * DO NOT CHANGE THIS ORDER!
	 */
	public function __construct()
	{
		$this->import('BackendUser', 'User');
		parent::__construct();

		$this->User->authenticate();
		\Contao\System::loadLanguageFile('default');
	}


	/**
	 * Run the controller and parse the template
	 */
	public function run()
	{
		$objSession = \Contao\Session::getInstance();
		if( version_compare(VERSION, '4.4', '>=') )
		{
			$objSession = \Contao\System::getContainer()->get('session');
		}

		$this->Template = new \Contao\BackendTemplate('be_iconpicker');
		$this->Template->main = '';

		// Ajax request
		if ($_POST && \Contao\Environment::get('isAjaxRequest'))
		{
			$this->objAjax = new \Contao\Ajax(\Contao\Input::post('action'));
			$this->objAjax->executePreActions();
		}
		
		$strTable = \Contao\Input::get('table');
		$strField = \Contao\Input::get('field');

		// Define the current ID
		define('CURRENT_ID', (\Contao\Input::get('table') ? \Contao\Session::getInstance()->get('CURRENT_ID') : \Contao\Input::get('id')));
		
		$this->loadDataContainer($strTable);
		$strDriver = 'DC_' . $GLOBALS['TL_DCA'][$strTable]['config']['dataContainer'];
		$objDca = new $strDriver($strTable);
		
		// AJAX request
		if ($_POST && \Contao\Environment::get('isAjaxRequest'))
		{
		   $this->objAjax->executePostActions($objDca);
		}
		
		$objSession->set('iconPickerRef', \Contao\Environment::get('request'));

		// Prepare the widget
		$objIconPicker = new $GLOBALS['BE_FFL']['iconPicker'](array(
			'strId'    => $strField,
			'strTable' => $strTable,
			'strField' => $strField,
			'strName'  => $strField,
			'varValue' => array_filter(explode(',', \Contao\Input::get('value')))
		), $objDca);
		
		$this->Template->main = $objIconPicker->generate();
		$this->Template->theme = \Contao\Backend::getTheme();
		$this->Template->base = \Contao\Environment::get('base');
		$this->Template->language = $GLOBALS['TL_LANGUAGE'];
		$this->Template->title = \specialchars($GLOBALS['TL_LANG']['MSC']['fontIconPicker']);
		$this->Template->charset = $GLOBALS['TL_CONFIG']['characterSet'];
		$this->Template->addSearch = true;
		$this->Template->search = $GLOBALS['TL_LANG']['MSC']['search'];
		$this->Template->action = \ampersand(\Contao\Environment::get('request'));
		$this->Template->manager = $GLOBALS['TL_LANG']['MSC']['pageManager'];
		$this->Template->managerHref = 'contao/main.php?do=page&amp;popup=1';
		
		$this->Template->value = $objSession->get('icon_selector_search');
		$this->Template->containerId = 'tl_listing';
		if(version_compare(VERSION, '3.5','>='))
		{
			$this->Template->containerId = 'tl_select';
		}
		
		$GLOBALS['TL_CONFIG']['debugMode'] = false;
		
		$this->Template->output();
	}
}
