<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2013, Premium Contao Webworks, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		revolutionslider
 * @link		http://contao.org
 */

/**
 * Namespace
 */
namespace RevolutionSlider\Core;

/**
 * Imports
 */
use \RevolutionSlider\Core\SlideFactory as SlideFactory;
use \RevolutionSlider\Core\Hooks as Hooks;

/**
 * Class file
 * RevolutionSlider Core Class
 * Provide methods to initialize a slider and add necessary scripts to the page
 */
class RevolutionSlider extends \Frontend
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'revoslider_default';
	
	/**
	 * Data Array
	 * @var array
	 */
	protected $arrData = array();
	
	/**
	 * Store information about parent element (cte, mod, ...)
	 * @var array
	 */
	protected $arrParent = array();

	/**
	 * Slides array
	 * @var array
	 */
	protected $arrSlides = array();

	/**
	 * Create new instance
	 * @param array
	 */
	public function __construct($arrData=array())
	{
		if(!empty($arrData))
		{
			$this->arrData = $arrData;
		}
	}

	/**
	 * Set Data
	 * @param array
	 */
	public function setData($arrData)
	{
		$this->arrData = $arrData;
	}

	/**
	 * Getters
	 * @param string
	 */
	public function get($strKey)
	{
		if(isset($this->$strKey))
		{
			return $this->$strKey;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			return $this->arrData[$strKey];
		}
		else
		{
			return null;
		}
		
	}
	
	/**
	 * Setters
	 * @param string
	 * @param mixed
	 */
	public function set($strKey, $varValue)
	{
		if(isset($this->$strKey))
		{
			$this->$strKey = $varValue;
		}
		else if(array_key_exists($strKey, $this->arrData))
		{
			$this->arrData[$strKey] = $varValue;
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * Generate a slider and return html and javascript
	 * @return string	html
	 */
	public function render()
	{
		global $objPage;
		$strLanguage = str_replace('-','_',$objPage->language);
		
		// slider template
		if($this->arrData['sliderTemplate'] != $this->strTemplate)
		{
			$this->strTemplate = $this->arrData['sliderTemplate'];
		}
		$objTemplate = new \FrontendTemplate($this->strTemplate);
		$objTemplate->setData($this->arrData);
		
		$arrCssId = deserialize($this->get('cssID'));
		$arrClass = explode(' ', $arrCssId[1]);
		$arrClass[] = $this->get('sliderStyle');
		$arrClass[] = 'tp-banner';
		
		if(in_array($this->get('sliderStyle'), array('responsive','boxed')))
		{
			$arrClass[] = 'fullwidthbanner-container';
		}
		else if(in_array($this->get('sliderStyle'), array('fullscreen')))
		{
			$arrClass[] = 'fullscreen-container';
		}
		
		$arrParent = $this->get('arrParent');
		
		//-- generate an unique css class or use ID as jquery selector
		$strSelector = 'my_revolutionslider_'.$this->get('id');
		
		if($arrParent['id'] > 0)
		{
			$strSelector = 'my_revolutionslider_'.$arrParent['id'];
		}
		
		if(strlen($arrCssId[0]) > 0)
		{
			$objTemplate->sliderID = 'id="'.$arrCssId[0].'"';
			$strSelector = '#'.$arrCssId[0];
		}
		else
		{
			$arrClass[] = $strSelector;
			$strSelector = '.'.$strSelector;
		}
		
		// use css id from content elemenet
		if($arrParent['cssID'])
		{
			$arrCssId = deserialize($arrParent['cssID']);
			if(strlen($arrCssId[0]) > 0)
			{
				$objTemplate->sliderID = 'id="revolutionslider_'.$arrCssId[0].'"';
				$strSelector = '#revolutionslider_'.$arrCssId[0];
			}
		}
		//--
		
		//-- slides
		$arrSlides = array();
		if(!empty($this->arrSlides) && is_array($this->arrSlides))
		{
			foreach($this->arrSlides as $i => $objSlide)
			{
				$arrRow = $objSlide->get('arrData');
				
				// classes
				$class =array('slide','slide_'.$i);
				$i%2 == 0 ? $class[] = 'even' : $arrClass[] = 'odd';
				if($i == 0) $class[] = 'first';
				if($i == count($this->arrSlides) - 1) $class[] = 'last';
				
				$cssID = deserialize($objSlide->get('cssID'));
				$class = array_merge(explode(' ', $cssID[1]),$class);
				
				// slide <li> attributes
				$arrAttributes = array();
				if(count($objSlide->get('arrAttributes')) > 0)
				{
					foreach($objSlide->get('arrAttributes') as $attr => $value)
					{
						$value = $this->replaceInsertTags($value);
						$arrAttributes[] = $attr.'="'.$value.'"';
					}
				}
				
				//-- content	
				$arrContent = $objSlide->get('arrContent');
				
				// fallback
				if(!$objSlide->get('background'))
				{
					$objSlide->set('background','image');
				}
				
				// inject main slide image on first position
				if($objSlide->get('background') == 'image')
				{
					$objFile = \FilesModel::findByPk($objSlide->get('singleSRC'));
					$arrMeta = deserialize($objFile->meta);
					$size = deserialize($objSlide->get('size'));
					$src = $objFile->path;
					$file = \Controller::getImage($src,$size[0],$size[1],$size[2]);
					$title = $arrMeta[$strLanguage]['title'];
					
					if($arrRow['lazyload'])
					{
						$dummy = REVOLUTIONSLIDER_PATH.'/assets/img/dummy.png';
						$dummy = $this->replaceInsertTags('{{image::'.$dummy.'?class=slideimage}}');
						
						$image = str_replace('>',' data-lazyload="'.$file.'" title="'.$title.'">', $dummy);
					}
					else
					{
						if(strlen($file) > 0)
						{
							$image = \Image::getHtml($file,$title,'title="'.$title.'"');
						}
					}
					
					$bgattr = array
					(
						'data-bgfit' 		=> $objSlide->get('data_bgfit') ? $objSlide->get('data_bgfit') : 'cover',
						'data-bgposition' 	=> $objSlide->get('data_bgposition') ? $objSlide->get('data_bgposition') : 'center center',
						'data-bgrepeat' 	=> $objSlide->get('data_bgrepeat') ? 'repeat' : 'no-repeat',
					);
					
					// add ken burns options
					if($objSlide->get('kenburns'))
					{
						$bgattr['data-kenburns'] = 'on';
						$data_scale = deserialize($objSlide->get('data_bgscale'));
			
						$bgattr['data-bgposition'] = $objSlide->get('data_bgposition');
						$bgattr['data-bgpositionend'] = $objSlide->get('data_bgposition_OUT');
						$bgattr['data-bgfit'] = (strlen($data_scale[0] > 0) ? $data_scale[0] : '100');
						$bgattr['data-bgfitend'] = (strlen($data_scale[1] > 0) ? $data_scale[1] : '100');
						$bgattr['data-duration'] = $objSlide->get('data_duration') * 1000;
						$bgattr['data-ease'] = $objSlide->get('data_ease');
					}
					
					$strbgattr = '';
					foreach($bgattr as $k => $v)
					{
						$strbgattr .= $k.'="'.$v.'" ';
					}
					$strbgattr = substr($strbgattr,0,-1);
					
					$image = str_replace('>',' '.$strbgattr.'>', $image);
				}
				// colored bg
				else if($objSlide->get('background') == 'colored')
				{
					$color = $objSlide->get('data_bgcolor') ? '#'.$objSlide->get('data_bgcolor') : 'transparent';
					$image = '<img src="'.REVOLUTIONSLIDER_PATH.'/assets/img/transparent.png" alt="" style="background-color:'.$color.'" data-bgfit="cover" data-bgposition="left top" data-bgrepeat="no-repeat">';
				}
				else if($objSlide->get('background') == 'transparent')
				{
					$image = '<img src="'.REVOLUTIONSLIDER_PATH.'/assets/img/transparent.png" alt="" data-bgfit="cover" data-bgposition="left top" data-bgrepeat="no-repeat">';
				}
				
				// inset main image on first position
				array_insert($arrContent,0,$image);
				$arrSlides[$i]['id']			= $objSlide->get('id');
				$arrSlides[$i]['content'] 		= $arrContent;
				$arrSlides[$i]['attributes']	= implode(' ', $arrAttributes);
				$arrSlides[$i]['class']			= implode(' ', $class);
				$arrSlides[$i]['cssID']			= (strlen($cssID[0]) ? 'id="'.$cssID[0].'"' : '');
			}
		}
		
		// call hook
		$arrSlides = Hooks::getInstance()->callAddSlideHook($arrSlides,$this);
		if(empty($arrSlides))
		{
			$arrClass[] = 'empty';
			// add status messages
			$objTemplate->emptyMsg = $GLOBALS['TL_LANG']['REVOLUTIONSLIDER']['empty'];
			
			return $objTemplate->parse();
		}
		
		$objTemplate->slides = $arrSlides;
		//--
		
		//-- js template
		$objJsTemplate = new \FrontendTemplate($this->arrData['jsTemplate']);
		$objJsTemplate->setData($this->arrData);
		$objJsTemplate->selector = $strSelector;
		$objJsTemplate->stopOnHover = ($this->get('stopOnHover') ? 'on' : 'off');
		$objJsTemplate->delay = ($this->arrData['delay'] ? $this->arrData['delay'] * 1000 : 0);
		
		$arrSize = deserialize($this->arrData['sliderSize']);
		$objJsTemplate->startWidth = ($arrSize[0] ? $arrSize[0] : 0);
		$objJsTemplate->startHeight = ($arrSize[1] ? $arrSize[1] : 0);
		$objJsTemplate->startWithSlide = ($this->get('startWithSlide') > 0 ? $this->get('startWithSlide') : 0);
		
		// modes
		$objJsTemplate->fullscreen = false;
		$objJsTemplate->fullwidth = true;
		switch($this->get('sliderStyle'))
		{
			case 'fullscreen': 
				$objJsTemplate->fullscreen = true;
				$objJsTemplate->fullwidth = false;
				break;
			default:
				break;
		}
		
		// navigation type
		if(!$this->get('addNavigation') || !$this->get('navigationType'))
		{
			$objJsTemplate->navigationType = 'none';
		}
		else
		{
			$tmp = explode('_', $this->get('navigationType'));
			switch($this->get('navigationType'))
			{
				case 'preview1':
				case 'preview2':
				case 'preview3':
				case 'preview4':
					$objJsTemplate->navigationType = 'none';
					$objJsTemplate->navigationStyle = $this->get('navigationType');
					break;
				default:
					$objJsTemplate->navigationType = $tmp[0];
					$objJsTemplate->navigationStyle = $tmp[1];
					break;
			}
		}
		
		// thumbnails
		if($this->get('addThumbs'))
		{
			$arrSize = deserialize($this->get('thumbSize'));
			$objJsTemplate->thumbWidth = ($arrSize[0] ? $arrSize[0] : 0);
			$objJsTemplate->thumbHeight = ($arrSize[1] ? $arrSize[1] : 0);
		}
		
		// arrow navigation
		if(!$this->get('addArrows'))
		{
			$objJsTemplate->arrowStyle = 'none';
		}
		
		// shuffle
		$objJsTemplate->shuffle = $this->get('shuffle');
	
		//overlay
		$objJsTemplate->overlay = $this->get('overlay');
		
		// loop settings
		$objJsTemplate->stopAtSlide = -1;
		if($this->get('stopAtSlide') >= 0 && $this->get('stopAtSlide') != '')
		{
			$objJsTemplate->stopAtSlide = $this->get('stopAtSlide');
		}
		$objJsTemplate->stopAfterLoops = -1;
		if($this->get('stopAfterLoops') >= 0 && $this->get('stopAfterLoops') != '')
		{
			$objJsTemplate->stopAfterLoops = $this->get('stopAfterLoops');
		}
		
		// parse js template
		$objTemplate->javascript = $objJsTemplate->parse();
		
		// add classes
		$objTemplate->class = implode(' ', $arrClass);
		
		// template vars
		$arrSize = deserialize($this->arrData['sliderSize']);
		$objTemplate->startWidth = $arrSize[0];
		$objTemplate->startHeight = $arrSize[1];
		
		return $objTemplate->parse();
	}
	
	/**
	 * Set Template
	 * @param string
	 */
	public function setTemplate($strTemplate)
	{
		$this->strTemplate = $strTemplate;
	}
	
	/**
	 * Find and set all slide objects for this slider and return as array
	 * @param integer
	 * @return array
	 */
	public function findSlides($intId)
	{
		$this->arrSlides = SlideFactory::findAllBySlider($intId);		
		return $this->arrSlides;
	}
	
	/**
	 * Return all transition options as array
	 * @return array
	 */
	public function getTransitions()
	{
		$arrReturn = array(
			'boxslide','boxfade', 'slotzoom-horizontal', 'slotzoom-vertical',
			'slotslide-horizontal', 'slotslide-vertical', 
			'slotfade-horizontal','slotfade-vertical', 
			'slideleft', 'slideright', 'slideup' ,'slidedown',
			'slidehorizontal', 'slidevertical',	
			'fade',	'random'
		);
		
		// new regular
		array_push($arrReturn,
			'fadefromright','fadefromleft','fadefromtop','fadefrombottom',
			'fadetoleftfadefromright','fadetorightfadefromleft','fadetotopfadefrombottom','fadetobottomfadefromtop',
			'parallaxtoright','parallaxtoleft','parallaxtotop','parallaxtobottom',
			'scaledownfromright','scaledownfromleft','scaledownfromtop','scaledownfrombottom',
			'zoomout','zoomin',
			'slotzoom-horizontal','slotzoom-vertical',
			'random-static'
		);
	
		// premiums
		$arrPremiums = array
		(
			 'curtain-1', 'curtain-2', 'curtain-3',
			 '3dcurtain-vertical','3dcurtain-horizonal',
			 'cube','cube-horizontal',
			 'papercut',
			 'flyin',
			 'turnoff','turnoff-vertical',
			 'random-premium'
		);
		
		// HOOK allow more transitions via global var
		if(!empty($GLOBALS['REVOLUTIONSLIDER']['TRANSITIONS']) && is_array($GLOBALS['REVOLUTIONSLIDER']['TRANSITIONS']))
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['TRANSITIONS']);
		}
		
		$arrReturn = array_unique(array_merge($arrReturn, $arrPremiums));

		// order
		sort($arrReturn);

		return $arrReturn;
	}
	
	/**
	 * Return easing options as array
	 * @return array
	 */	
	public function getTransitionEasings()
	{
		$arrReturn = array
		(
			 'easeInBack','easeOutBack', 'easeOutBack', 'easeInOutBack',
			 'easeInQuad', 'easeOutQuad', 'easeInOutQuad',  
			 'easeOutQuart', 'easeInOutQuart',
			 'easeInCubic', 'easeOutCubic', 'easeInOutCubic', 'easeInQuart',
			 'easeInQuint', 'easeOutQuint', 'easeInOutQuint',
			 'easeInSine', 'easeOutSine', 'easeInOutSine', 
			 'easeInExpo', 'easeOutExpo', 'easeInOutExpo',
			 'easeInCirc', 'easeOutCirc', 'easeInOutCirc', 
			 'easeInElastic', 'easeOutElastic','easeInOutElastic',  
			 'easeInBounce', 'easeOutBounce', 'easeInOutBounce',
		);
		
		// new easings
		#$arrReturn['new'] = array
		#(
		#	'Linear.easeNone Power0.easeIn (linear)',
		#	'Power0.easeInOut (linear)',
		#	'Power0.easeOut (linear)','Power1.easeIn', 'Power1.easeInOut',
		#	'Power1.easeOut',
		#	'Power2.easeIn', 'Power2.easeInOut',
		#	'Power2.easeOut', 'Power3.easeIn', 
		#	'Power3.easeInOut', 'Power3.easeOut', 'Power4.easeIn', 'Power4.easeInOut', 'Power4.easeOut',
		#	'Quad.easeIn (same as Power1.easeIn)','Quad.easeInOut (same as Power1.easeInOut)',
		#	'Quad.easeOut (same as Power1.easeOut)', 'Cubic.easeIn (same as Power2.easeIn)', 'Cubic.easeInOut (same as Power2.easeInOut)', 'Cubic.easeOut (same as Power2.easeOut)',
		#	'Quart.easeIn (same as Power3.easeIn)',
		#	'Quart.easeInOut (same as Power3.easeInOut)',
		#	'Quart.easeOut (same as Power3.easeOut)',
		#	'Quint.easeIn (same as Power4.easeIn)',
		#	'Quint.easeInOut (same as Power4.easeInOut)',
		#	'Quint.easeOut (same as Power4.easeOut)',
		#	'Strong.easeIn (same as Power4.easeIn)', 'Strong.easeInOut (same as Power4.easeInOut)',
		#	'Strong.easeOut (same as Power4.easeOut)',
		#	'Back.easeIn','Back.easeInOut',	'Back.easeOut',
		#	'Bounce.easeIn','Bounce.easeInOut', 'Bounce.easeOut', 
		#	'Circ.easeIn', 'Circ.easeInOut', 'Circ.easeOut', 
		#	'Elastic.easeIn', 'Elastic.easeInOut', 'Elastic.easeOut', 'Expo.easeIn', 'Expo.easeInOut', 'Expo.easeOut',
		#	'Sine.easeIn', 'Sine.easeInOut', 'Sine.easeOut', 'SlowMo.ease'
		#);
		
		// HOOK allow more easings via global var
		if(!empty($GLOBALS['REVOLUTIONSLIDER']['EASINGS']) && is_array($GLOBALS['REVOLUTIONSLIDER']['EASINGS']))
		{
			$arrReturn['custom'] = $GLOBALS['REVOLUTIONSLIDER']['EASINGS'];
		}
		
		return $arrReturn;
	}
	
	/**
	 * Return start animations classes as array
	 * @return array
	 */	
	public function getStartAnimationClasses()
	{
		$arrReturn = array
		(
			'sft',	//- Short from Top
			'sfb',	// - Short from Bottom
			'sfr',	// - Short from Right
			'sfl',	// - Short from Left
			'lft',	// - Long from Top
			'lfb',	// - Long from Bottom
			'lfr',	// - Long from Right
			'lfl',	// - Long from Left
			'fade',	// - fading
			'randomrotate',	//- Fade in, Rotate from a Random position and Degree
		);
				
		// new transitions
		array_push($arrReturn,
			'skewfromleft','skewfromright',
			'skewfromleftshort','skewfromrightshort',
			'customin'
		);

		// HOOK allow more animation classes via global var
		if(!empty($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']) && is_array($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']))
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']);
		}
		
		sort($arrReturn);
		
		return array_unique($arrReturn);
	}
	
	/**
	 * Return end animations classes as array
	 * @return array
	 */	
	public function getEndAnimationClasses()
	{
		$arrReturn = array
		(
			'stt',	// - Short to Top
			'stb',	// - Short to Bottom
			'str',	// - Short to Right
			'stl',	// - Short to Left
			'ltt',	// - Long to Top
			'ltb',	// - Long to Bottom
			'ltr',	// - Long to Right
			'ltl',	//- Long to Left
			'fadeout',	// - fading
			'randomrotateout',	//- Fade in, Rotate from a Random position and Degree
		);
		
		// new transitions
		array_push($arrReturn,
			'skewfromleft','skewfromright',
			'skewfromleftshort','skewfromrightshort',
			'customout'
		);
		
		// HOOK allow more animation classes via global var
		if(!empty($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']) && is_array($GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']))
		{
			$arrReturn = array_merge($arrReturn, $GLOBALS['REVOLUTIONSLIDER']['ANIMATIONS']['START']);
		}
		
		sort($arrReturn);
	
		return array_unique($arrReturn);
	}

}