<?php

/**
 * Contao Open Source CMS
 * 
 * Copyright (C) 2005-2013 Leo Feyer
 * 
 * @copyright	Tim Gatzky 2017, Premium Contao Themes
 * @author		Tim Gatzky <info@tim-gatzky.de>
 * @package		pct_themer
 */

/**
 * Namespace
 */
namespace PCT\Themer;

/**
 * Imports
 */
use Contao\Input;
use Contao\System;
use Contao\Message;
use Contao\Environment;
use Contao\Config;

/**
 * Class file
 * DemoInstaller
 */
class DemoInstaller extends \Contao\BackendModule
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'be_pct_demoinstaller';


	/**
	 * Generate the module
	 *
	 * @throws \Exception
	 */
	protected function compile()
	{
		System::loadLanguageFile('pct_demoinstaller');
		
		// @var object Session
		$objSession = \Contao\Session::getInstance();
		$arrSession = $objSession->get('pct_demoinstaller');
		if(!is_array($arrSession['categories']))
		{
			$arrSession['categories'] = array();
		}
		if(!is_array($arrSession['installed']))
		{
			$arrSession['installed'] = array();
		}

		
		$this->Template->content = '';
		$this->Template->href = $this->getReferer(true);
		$this->Template->title = specialchars($GLOBALS['TL_LANG']['MSC']['backBTTitle']);
		$this->Template->button = $GLOBALS['TL_LANG']['MSC']['backBT'];
		
		if(empty($GLOBALS['PCT_THEMER']['THEMES']) || !is_array($GLOBALS['PCT_THEMER']['THEMES']))
		{
			$this->Template->content = 'No demos available';
			return;
		}
		
		$arrCategories = array();
		$arrThemes = array();
		$arrSkipped = array();
		$i = 0;
		foreach($GLOBALS['PCT_THEMER']['THEMES'] as $k => $v)
		{
			// skip demos
			if($v['installer'] === false)
			{
				continue;
			}
			
			if(!is_array($v['category']))
			{
				$v['category'] = array_filter(explode(',',$v['category']));
			}
			
			// collect categories
			foreach($v['category'] as $cat)
			{
				$arrCategories[$cat] = $cat;
			}
			
			// filter active
			if(count($arrSession['categories']) > 0 && !array_intersect($arrSession['categories'],$v['category']))
			{
				continue;
			}
			
			// remove eclipse from name
			$filename = str_replace('eclipse_', '', $k);
			$thumb = strlen($v['thumb']) > 0 ? $v['thumb'] : PCT_THEMER_PATH.'/assets/img/demoinstaller/demo_'.$filename.'.jpg';
			$img = '';
			
			if(file_exists(TL_ROOT.'/'.$thumb))
			{
				// cached
				#$img = \Contao\Controller::replaceInsertTags('{{image::'.$thumb.'?width=200&height=200&alt='.$v['label'].'}}');#\Image::getHtml($thumb,$v['label']);
				// no cached
				$img = \Contao\Image::getHtml($thumb,$v['label']);
			}
			
			// skip when there is no image
			if(strlen($img) < 1)
			{
				$i--;
				
				if(Config::get('debugMode'))
				{
					$arrSkipped[] = $k;
				}
				
				continue;
			}
			
			$arrClass = array('item',$k);
			if($i == 0) {$arrClass[] = 'first';}
			if($i >= count($GLOBALS['PCT_THEMER']['THEMES']) -1 ) {$arrClass[] = 'last';}
			$i%2 == 0 ? $arrClass[] = 'even' : $arrClass[] = 'odd';
			
			// collect output
			$arrThemes[$k] = array
			(
				'label'	=> $v['label'] ?: $k,
				'id' 	=> $k,
				'src'	=> $thumb,
				'img'	=> $img,
				'class'	=> implode(' ', $arrClass),
			);
			
			if(array_key_exists($k, $arrSession['installed']) && \Contao\PageModel::findByPk($arrSession['installed'][$k]) !== null)
			{
				$arrThemes[$k]['installed'] = true;
				
				$feRedirect = Environment::get('script').'?do=feRedirect&page='.$arrSession['installed'][$k];
				if(version_compare(VERSION, '4', '>='))
				{
					$feRedirect = Environment::get('base').'contao'.'?do=feRedirect&page='.$arrSession['installed'][$k];
				}
				
				$arrThemes[$k]['feRedirect'] = $feRedirect;
			}
			
			$i++;
		}
		
		// log skipped themes
		if(Config::get('debugMode') && count($arrSkipped) > 0)
		{
			System::log('Themer: Skipped demo: '.implode(', ', $arrSkipped),__METHOD__,TL_GENERAL);
		}
			
		
		// stort arrays
		$arrCategories = array_filter(array_unique($arrCategories));
		asort($arrCategories);
		
		#$arrDemos = array_filter(array_unique($arrDemos));
		ksort($arrThemes);
		
		// check if there is more than one theme record
		
		// @var object	ModelCollection
		$objContaoThemes = \Contao\ThemeModel::findAll();
		if($objContaoThemes === null)
		{
			$this->Template->messages = 'No theme installed yet.';
			return;
		}
		
		// more than one theme record to choose from
		if(count($objContaoThemes->getModels()) > 1)
		{
			$intContaoTheme = $arrSession['contao_theme'];
			
			// contao theme select
			$arrField = array
			(
				'inputType'	=> 'select',
				'label'		=> &$GLOBALS['TL_LANG']['pct_demoinstaller']['contao_theme'],
				'foreignKey'=> 'tl_theme.name',
				#'eval'		=> array('includeBlankOption'=>true),
				'attributes'=>'onchange="asdf"',
			);
			
			$strClass = $GLOBALS['BE_FFL'][$arrField['inputType']];
			$arrAttributes = $strClass::getAttributesFromDca($arrField,'contao_theme',$arrSession['contao_theme']);
			$objWidget = new $strClass($arrAttributes);
			$objWidget->__set('onchange','this.form.submit()');
			$this->Template->contaoThemeWidget = $objWidget->parse();
			
			if(Input::post('FORM_SUBMIT') == 'tl_filter' && Input::post('contao_theme') > 0)
			{
				$arrSession['contao_theme'] = Input::post('contao_theme');
				// set session
				$objSession->set('pct_demoinstaller',$arrSession);
				// reload
				$this->reload();
			}
		}
		else
		{
			$intContaoTheme = $objContaoThemes->id;
		}
		
		// store filter
		if(Input::post('FORM_SUBMIT') == 'tl_filter')
		{
			$arrSession['categories'] = Input::post('categories');
			// set session
			$objSession->set('pct_demoinstaller',$arrSession);
			// reload
			$this->reload();
		}
		
		// run install
		if(Input::post('FORM_SUBMIT') == 'pct_demoinstaller' && Input::post('THEME') != '' && Input::post('install') != '')
		{
			// run installation, returns the page id
			$intPage = $this->install(Input::post('THEME'),$intContaoTheme);
				
			if($intPage > 0)
			{
				Message::addInfo($GLOBALS['TL_LANG']['pct_themer']['import_success']);
			}
			else
			{	
				Message::addError('An error occurred while importing the demo template');
			}

			// mark as installed
			$arrSession['installed'][Input::post('THEME')] = $intPage;
			
			// set session
			$objSession->set('pct_demoinstaller',$arrSession);
			// reload
			$this->reload();
		}
		
		// filter form
		$this->Template->action = Environment::get('request');
		
		// category filter select
		$arrField = array
		(
			'inputType'	=> 'select',
			'label'		=> &$GLOBALS['TL_LANG']['pct_demoinstaller']['categories'],
			'options'	=> $arrCategories,
			'reference'	=> $GLOBALS['TL_LANG']['pct_demoinstaller']['categories'],
			'eval'		=> array('includeBlankOption'=>true,'chosen'=>true,'multiple'=>true),
		);
		
		$strClass = $GLOBALS['BE_FFL'][$arrField['inputType']];
		$arrAttributes = $strClass::getAttributesFromDca($arrField,'categories',$arrSession['categories'] ?: array());
		$objWidget = new $strClass($arrAttributes);
		$this->Template->categoryWidget = $objWidget->parse();
		$this->Template->themes = $arrThemes;
		$this->Template->messages = Message::generate();
		
		$this->Template->showLabel = $GLOBALS['TL_LANG']['pct_demoinstaller']['submit_show'] ?: 'Show me';
		$this->Template->customizeLabel = $GLOBALS['TL_LANG']['pct_demoinstaller']['submit_customize'] ?: 'Customize';
	}
	
	
	/**
	 * Install a demo
	 * @param string	Name of the demo
	 * @param integer	Optional the ID of the contao theme record to install frontend modules in
	 * @return integer	Id of the new root page created or -1 for an error
	 */
	public function install($strTheme,$intContaoTheme=0)
	{
		if(strlen($strTheme) < 1 ||$intContaoTheme < 1 || !is_array($GLOBALS['PCT_THEMER']['THEMES'][$strTheme]))
		{
			return -1;
		}
		
		$objDatabase = \Contao\Database::getInstance();
		
		$arrTheme = $GLOBALS['PCT_THEMER']['THEMES'][$strTheme];
		$strLayout = $arrTheme['layout'] ?: $GLOBALS['PCT_THEMER']['defaultLayout'];
		
		// find the layout
		$objLayout = $objDatabase->prepare("SELECT * FROM tl_layout WHERE pid=? AND name=?")->limit(1)->execute($intContaoTheme,$strLayout);
		
		$time = time();
		
		// create a new root page
		$arrSet = array
		(
			'tstamp'	=> $time,
			'type'		=> 'root',
			'language'	=> 'de',
			'title'		=> $arrTheme['label'] ?: $strTheme,
			'redirect'	=> 'permanent',
			'published'	=> 1,
			'pct_theme'	=> $strTheme,
			'pct_theme_cto'	=> $intContaoTheme
		);
		
		// predefine a page layout if found
		if($objLayout->id > 0)
		{
			$arrSet['includeLayout'] = 1;
			$arrSet['layout'] = $objLayout->id;
		}
		
		$objInsert = $objDatabase->prepare("INSERT INTO tl_page %s")->set($arrSet)->execute();
		
		// new root id
		$intRoot = $objInsert->__get('insertId');
		
		// update alias
		$objDatabase->prepare("UPDATE tl_page %s WHERE id=?")->set(array('alias' => $strTheme.'_'.$intRoot))->execute($intRoot);
		
		// @var \PCT\Themer
		$objThemer = new \PCT\Themer;
		
		// run import
		if( $objThemer->import($intRoot) )
		{
			return $intRoot;
		}

		return -1;
	}
}